/* $Id: main.c,v 1.1.1.1 2003/03/14 15:20:17 roman Exp $ */
/* written by Roman R. Stocsits, 2003 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "utils.h"
#include "structs.h"

struct SEQdata *SEQ;
struct FILEdata *inputF;
long count;

void start(int argc, char **argv);
void use_me_this_way(short ret);
void findORF();
void makePS();
void infoFile();
void help(short ret);
short decideFN(char FN[256], char CTN[8]);
short readGBF(char FN[256], char CTN[8], char *line, FILE *fp, short is_fp);
char *get_seq(FILE *fp, char *line, long len, short is_fp);
short readPears(char FN[256], char CTN[8], char *line, FILE *fp, short is_fp);
short write2SEQ1(char FN[256], char CTN[8], char seqname[256],
		 char *seq, long len, struct CDSdata* ORF,
		 struct CDSdata *exon);
void mergeJOIN();
int align(); 
void pretranslate();
void encode();
void output();
void check();

/*--------------------------------------------------------------------------*/

int main(int argc, char **argv){

  int c;
  
  count=0;
  inputF=(struct FILEdata*)space(100*sizeof(struct FILEdata));
  SEQ=(struct SEQdata*)space(300*sizeof(struct SEQdata));
  start(argc, argv);
  free(inputF);
  findORF();
  makePS();
  infoFile();
  mergeJOIN();
  pretranslate();
  encode();
  c = align();
  output(c);
  check();
  fprintf(stdout, "See 'cluster.txt' and 'info.ral' for information about profiles.\n");
  return(0);
}

/*--------------------------------------------------------------------------*/

void start(int argc, char **argv){
  
  int i, dir;
  short is_tty, ok=0, a=0;
  char currCTN[8], dummy[500];
  
  is_tty=isatty(0);
  strcpy(currCTN, "univ\0");

  if(argc == 1) help(1);
  for(i=1;i<argc;i++){
    dir = strlen(argv[i]);
    if(argv[i][dir-1] == '/'){
      fprintf(stdout, "No files specified. Please enter file names!\n");
      exit(31);
    }
  }
  for(i=1;i<argc;i++){
    if(argv[i][0]=='-'){
      switch(argv[i][1]){
      case 'h': help(1);
      case 'c':
	if(argv[++i]){
	  sscanf(argv[i], "%s", &dummy[0]);
	  if((!strcmp(dummy, "univ")) || (!strcmp(dummy, "acet")) ||
	     (!strcmp(dummy, "ccyl")) || (!strcmp(dummy, "tepa")) ||
	     (!strcmp(dummy, "eupl")) || (!strcmp(dummy, "mlut")) ||
	     (!strcmp(dummy, "mysp")) || (!strcmp(dummy, "mitocan")) ||
	     (!strcmp(dummy, "mitovrt")) || (!strcmp(dummy, "mitoart")) ||
	     (!strcmp(dummy, "mitoech")) || (!strcmp(dummy, "mitomol")) ||
	     (!strcmp(dummy, "mitoasc")) || (!strcmp(dummy, "mitonem")) ||
	     (!strcmp(dummy, "mitopla")) || (!strcmp(dummy, "mitoyea")) ||
	     (!strcmp(dummy, "mitoeua")) || (!strcmp(dummy, "mitopro"))){
	    strcpy(currCTN, dummy);
	    free(dummy);
	    if(!argv[i+1]){
	      if(argc > 3) help(5);
	      fprintf(stdout, "No input sequences. No alignment.\n");
	      exit(4);
	    }
	  }
	  else help(3);
	}
	else help(2);
	break;
      default: help(0);
      }
      continue;
    }
    else {
      strcpy(inputF[a].FN, argv[i]);
      strcpy(inputF[a].CTN, currCTN);
    }
    ok=decideFN(inputF[a].FN, inputF[a].CTN);
    a++;
  }
  if(!is_tty) {
    strcpy(inputF[a].CTN, currCTN);
    inputF[a].FN[0]='\0';
    ok=decideFN(inputF[a].FN, inputF[a].CTN);
  }
  if(!ok){
    fprintf(stdout, "Only one input sequence. No alignment.\n");
    exit(10);
  }
}

/*--------------------------------------------------------------------------*/

void help(short ret){
  
  char *date;

  date=time_stamp();
  fprintf(stderr, "\n%s\n", "* * * * * * code2aln * * * * * *");
  fprintf(stderr, "%s\n", "written by Roman Stocsits");
  fprintf(stderr, "date:  %s\n", date);
  use_me_this_way(ret);
}

/*--------------------------------------------------------------------------*/

void use_me_this_way(short ret){

  fprintf(stderr, "Nucleic Acid Sequence Alignments Based on\n");
  fprintf(stderr, "Amino Acid Sequence Information (Version 1.0)\n\n");
  fprintf(stderr, "Usage: code2aln (-c CTN) FN FN ... (-c CTN) FN FN ...\n");
  fprintf(stderr, "      FN: path(s) and file name(s) of GenBank file(s)\n");
  fprintf(stderr, "          or sequence file(s) in Pearson's format.\n");
  fprintf(stderr, "     CTN: abbreviated name of desired codon table\n");
  fprintf(stderr, "          (default: universal genetic code)\n");
  fprintf(stderr, "          every CTN is used for the following files,\n");
  fprintf(stderr, "          multiple CTN usage is possible.\n\n");
  fprintf(stderr, "The following codon tables are available:\n");
  fprintf(stderr, "    univ: universal genetic code (default)\n");
  fprintf(stderr, "    acet: Acetabularia\n");
  fprintf(stderr, "    ccyl: Candida cylindrica\n");
  fprintf(stderr, "    tepa: Tetrahymena, Paramecium,\n");
  fprintf(stderr, "          Oxytrichia, Stylonychia, Glaucoma\n");
  fprintf(stderr, "    eupl: Euplotes\n");
  fprintf(stderr, "    mlut: Micrococcus luteus\n");
  fprintf(stderr, "    mysp: Mycoplasma, Spiroplasma\n");
  fprintf(stderr, " mitocan: canonical mitochondrial code\n");
  fprintf(stderr, " mitovrt: Vertebrates -  mitochondrial code\n");
  fprintf(stderr, " mitoart: Arthropods -  mitochondrial code\n");
  fprintf(stderr, " mitoech: Echinoderms -  mitochondrial code\n");
  fprintf(stderr, " mitomol: Molluscs -  mitochondrial code\n");
  fprintf(stderr, " mitoasc: Ascidians -  mitochondrial code\n");
  fprintf(stderr, " mitonem: Nematodes -  mitochondrial code\n");
  fprintf(stderr, " mitopla: Plathelminths -  mitochondrial code\n");
  fprintf(stderr, " mitoyea: Yeasts -  mitochondrial code\n");
  fprintf(stderr, " mitoeua: Euascomycetes - mitochondrial code\n");
  fprintf(stderr, " mitopro: Protozoans - mitochondrial code\n");
  /* fprintf(stderr, "code2aln -h displays this help.\n"); */
  exit(ret);
}

/*--------------------------------------------------------------------------*/

short decideFN(char FN[256], char CTN[8]){

  FILE *fp;
  char *line;
  short ok=0, is_fp=0;
  
  if(FN[0]){
    fp=fopen(FN, "r");
    if(!fp){
      fprintf(stderr, "Could not open file '%s'.\n", FN);
      exit(4);
    }
    else is_fp=1;
  }
  else{
    fp=NULL;
    is_fp=0;
  }
  line=get_line(fp);
  if(line[0]=='>') ok=readPears(FN, CTN, line, fp, is_fp);
  else {
    if((!strncmp(line, "[Entrez", 7))||
       (!strncmp(line, "LOCUS", 5))||
       (!strncmp(line, "ID", 2))||
       (!isalpha(line[0])))    ok=readGBF(FN, CTN, line, fp, is_fp);
    else  ok=readPears(FN, CTN, line, fp, is_fp);
  }
  return(ok);
}

/*---------------------------------------------------------------------------*/

char *get_seq(FILE *fp, char *line, long len, short is_fp){

  char *seq;
  long j, k=0, n;
  
  
  if((!strncmp(line,"ORIGIN", 6))||(!strncmp(line, "SQ", 2))){
    free(line);
    line=get_line(fp);
  }
  if(len==0){
    fprintf(stderr,"Bad error! Failure of file reading.\n");
    exit(33);
  }
  seq=(char*)space((len+1)*sizeof(char));
  while(strncmp(line,"//",2)!=0){
    n=strlen(line);
    for (j=0;j<n;j++){
      if(isalpha(line[j])){
	seq[k]=toupper(line[j]);
	k++;
      }  
    }
    free(line);
    line=get_line(fp);
    continue;
  }
  seq[k]='\0'; 
  if(k!=len){
    fprintf(stderr,"Bad error! Failure of file reading.\n");
    exit(4);
  }
  return(seq);
}

/*--------------------------------------------------------------------------*/

short readPears(char FN[256], char CTN[8], char *line, FILE *fp, short is_fp){

  char *seq, seqname[256], revname[256];
  long n, m=0, k=0, i, len; 
  short ok=0;
  int x, y;
  struct CDSdata *orf=NULL;
  struct CDSdata *exon=NULL;
  
  seq=NULL;
  orf=(struct CDSdata*)space(120*sizeof(struct CDSdata)); 
  exon=(struct CDSdata*)space(120*sizeof(struct CDSdata));
  for(i=0;i<120;i++){
    exon[i].start=exon[i].stop=orf[i].start=orf[i].stop=0;
  }
  seqname[0]='\0';
  while(line){
    if(!strncmp(line, ">", 1)){
      sscanf(line, "%*[>] %s", seqname);
      seq=NULL;
      k=0;
      m=0;

      /* free(line); */
      line=get_line(fp);
    }
    n=strlen(line);
    m=m+n;
    if(seq){
      seq=(char*)realloc(seq, (m+1)*sizeof(char));
    }
    else {
      seq=(char*)space((n+1)*sizeof(char));
    }
    for(i=0;i<n;i++){
      if(isalpha(line[i])){
	seq[k] = toupper(line[i]);
	k++;
      }
    }
    seq[k]='\0';
    /* free(line); */
    line = get_line(fp);
    if(line){
      if(!strncmp(line, ">", 1)){
	len=strlen(seq);
	ok=write2SEQ1(FN, CTN, seqname, seq, len, orf, exon);
	free(seq);
      }
    }
  }
  len=strlen(seq);
  if(!seqname[0]){
    x = strlen(FN);
    y=-1;
    while(x){
      x--;
      y++;
      if(FN[x] != '/')  revname[y] = FN[x];
      else break;
    }
    if(!x)     revname[y+1] = '\0';
    else  revname[y] = '\0';
    x = strlen(revname);
    y=0;
    while(x){
      x--;
      seqname[y] = revname[x];
      y++;
    }
    seqname[y] = '\0';
  }
  
  ok=write2SEQ1(FN, CTN, seqname, seq, len, orf, exon);
  free(seq);
  if(is_fp)  fclose(fp);
  return(ok);
}

/*--------------------------------------------------------------------------*/

short write2SEQ1(char FN[256], char CTN[8], char seqname[256],
		 char *seq, long len, struct CDSdata *ORF,
		 struct CDSdata *exon){

  long ok;
  
  SEQ[count].no=count+1;
  SEQ[count].len=len;
  strcpy(SEQ[count].FN, FN);
  SEQ[count].seq=(char*)space((len+1)*sizeof(char));
  strcpy(SEQ[count].seq, seq);
  SEQ[count+1].seq=NULL;
  SEQ[count].seqname=(char*)space(257*sizeof(char));
  strcpy(SEQ[count].seqname, seqname);
  strcpy(SEQ[count].CTN, CTN);
  SEQ[count].ORF=ORF;
  SEQ[count].exon=exon;
  ok=count;
  count++;
  return(ok);
}

