/* $Id: scoring.c,v 1.1.1.1 2003/03/14 15:20:17 roman Exp $ */
/* written by Roman R. Stocsits, 2003 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "utils.h"
#include "structs.h"
#include "blosum62.h"
#include "scoretable.h"

extern struct SEQdata *SEQ;
extern struct ALNdata *aln;

int MscoreM(struct ALNdata tempaln[2], int c, int g, int h);
int MscoreIx(struct ALNdata tempaln[2], int c, int g, int h);
int MscoreIy(struct ALNdata tempaln[2], int c, int g, int h);

int scoreM(int a, int b, int c, int g, int h);
int scoreIx(int a, int b, int c, int g, int h);
int scoreIy(int a, int b, int c, int g, int h);

/* ----------------------------------------------------------------------- */

int scoreM(int a, int b, int c, int g, int h){
  
  int s = 0, sM, sIx, sIy, x=0, y=0, z=0;

  h=h-1;
  g=g-1;
  if(SEQ[a].letter[h].nucl == SEQ[b].letter[g].nucl)  s = NS_MATCH;
  else   s=NS_MISM;

  for(z=0; z<3; z++){
    if((SEQ[a].letter[h].aa[z]) && (SEQ[b].letter[g].aa[z])){
      if((isalpha(SEQ[a].letter[h].aa[z])) && (isalpha(SEQ[b].letter[g].aa[z]))){
	x=0;  y=0;
	while((x<23) && (SEQ[a].letter[h].aa[z] != AAorder[x]))      x++;
	while((y<23) && (SEQ[b].letter[g].aa[z] != AAorder[y]))      y++;
	if (x>22 || y>22) continue;
	s = s + (BLOSUM62[y][x] / 2);
      }
    }
  }
  sM = aln[c].SCmatrix[0][g][h] + s;
  sIx = aln[c].SCmatrix[1][g][h] + s;
  sIy = aln[c].SCmatrix[2][g][h] + s;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[0][g][h]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[0][g][h]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[0][g][h]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

int scoreIx(int a, int b, int c, int g, int h){
  
  int sM, sIx, sIy;
  
  sM = aln[c].SCmatrix[0][g][h-1] + GAP_OPEN;
  sIx = aln[c].SCmatrix[1][g][h-1] + GAP_EXT;
  sIy = aln[c].SCmatrix[2][g][h-1] + GAP_OPEN;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[1][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

int scoreIy(int a, int b, int c, int g, int h){

  int sM, sIx, sIy;

  sM = aln[c].SCmatrix[0][g-1][h] + GAP_OPEN;
  sIx = aln[c].SCmatrix[1][g-1][h] + GAP_OPEN;
  sIy = aln[c].SCmatrix[2][g-1][h] + GAP_EXT;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[2][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

int MscoreM(struct ALNdata tempaln[2], int c, int g, int h){
  
  int x=0, y=0, z=0, s=0, sM=0, sIx=0, sIy=0, i=0, k=0, m=0, rest;
  
  g=g-1;
  h=h-1;
  while(tempaln[0].alignment[x]){
    while(tempaln[1].alignment[y]){
      if(isalpha(tempaln[0].alignment[x][h].nucl) && isalpha(tempaln[1].alignment[y][g].nucl)){
	if(tempaln[0].alignment[x][h].nucl == tempaln[1].alignment[y][g].nucl){
	  s = s + NS_MATCH;
	}
	else   s = s + NS_MISM;
	for(i=0; i<3; i++){
	  if((tempaln[0].alignment[x][h].aa[i]) &&
	     (tempaln[1].alignment[y][g].aa[i])){
	    if((isalpha(tempaln[0].alignment[x][h].aa[i])) &&
	       (isalpha(tempaln[1].alignment[y][g].aa[i]))){
	      k=0;   m=0;
	      while((k<23) && (tempaln[0].alignment[x][h].aa[i] != AAorder[k]))   k++;  
	      while((m<23) && (tempaln[1].alignment[y][g].aa[i] != AAorder[m]))   m++;
	      if(k>22 || m>22) continue;
	      s = s + (BLOSUM62[m][k] / 2);
	    }
	  }
	}
      }
      y++;
      z++;
    }
    x++;
    y=0;
  }
  if(z){
    s = s*10;
    s = s/z;
    rest = s%10;
    s = s/10;
    if(rest >= 5)  s++;
  }
  sM = aln[c].SCmatrix[0][g][h] + s;
  sIx = aln[c].SCmatrix[1][g][h] + s;
  sIy = aln[c].SCmatrix[2][g][h] + s;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[0][g][h]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[0][g][h]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[0][g][h]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

int MscoreIx(struct ALNdata tempaln[2], int c, int g, int h){

  int sM, sIx, sIy;

  sM = aln[c].SCmatrix[0][g][h-1] + GAP_OPEN;
  sIx = aln[c].SCmatrix[1][g][h-1] + GAP_EXT;
  sIy = aln[c].SCmatrix[2][g][h-1] + GAP_OPEN;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[1][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

int MscoreIy(struct ALNdata tempaln[2], int c, int g, int h){

  int sM, sIx, sIy;
  
  sM = aln[c].SCmatrix[0][g-1][h] + GAP_OPEN;
  sIx = aln[c].SCmatrix[1][g-1][h] + GAP_OPEN;
  sIy = aln[c].SCmatrix[2][g-1][h] + GAP_EXT;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[2][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

